"""
addons/eventscripts/mysqlads/mysqlads.py
Version 1.02.000
(C) [#OMEGA] - K2

IMPORTANT

This plugin is designed to be used with
MySQLdb or pymysql

Agreement:
 
By using this file you agree to the following terms:

This file is delivered 'as-is'. 
I will not take resposibilty for any damage resulting of the use or not-use of this file. 

As for sharing permissions:
http://creativecommons.org/licenses/by-nc-nd/3.0/

If wish to alter the file/contents or reuse some parts please contact me.
"""

# ######## #
# Imports  #
# ######## #

# ES Libs
import es
import gamethread
import playerlib
import langlib
import usermsg

# Custom ES Libs
import pubcvarlib

# Python Libs
import re
import urllib2
import time
import random
import sys
from configobj import ConfigObj

# Custom Python Libs
try:
	import MySQLdb as mysql
	from MySQLdb import OperationalError
except:
	try:
		import pymysql as mysql
		from pymysql.err import OperationalError
	except:
		raise ImportError, "Failed to import MySQLdb or pymysql."

# ########## #
# Addon Info #
# ########## #

info = es.AddonInfo()
info.name        = "MySQL Advertisements"
info.version     = "1.02.000"
info.author      = "[#OMEGA] - K2"
info.url         = "http://www.german-slaughterhouse.de"
info.description = "MySQL Advertisements"
info.tags        = "mysqlads mysql advertisements serverads"

# ########### #
# Global Data #
# ########### #

# Load the Config
config = ConfigObj(es.getAddonPath("mysqlads") + '/config.ini')

# Load the Translation/String file
lang = langlib.Strings(es.getAddonPath("mysqlads") + '/mysqlads-strings.ini')	

selfmodule = __import__(__name__)

# ############# #
# Load & Unload #
# ############# #

def load():
	# Set Pub vars
	pubcvarlib.Register('mysqlads_ver', info.version)
	
	if config['MySQL']['sid'] == '-1':
		# Get IP and Port
		global ip,port
		
		port = es.getString('hostport')
		
		ip = re.search("[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}",urllib2.urlopen('http://www.whatismyip.de').read()).group(0)
	
	# Init DB
	global db
	db = Database()
	
	if db.data['servers']:
		gamethread.delayedname(db.data['servers']['interval'], 'mysqlads_adloop', AdLoop) 
	
def unload():
	# Cleanup pub vars
	pubcvarlib.Unregister('mysqlads_ver')
	
	# Disconnect from DB
	db.Disconnect()
	
	#
	gamethread.cancelDelayed('mysqlads_adloop') 

# ###### #
# Events #
# ###### #

def es_map_start(ev):
	db.SyncDB()
	# Restart the Loop
	gamethread.cancelDelayed('mysqlads_adloop') 
	gamethread.delayedname(db.data['servers']['interval'], 'mysqlads_adloop', AdLoop) 

def CustomEvent(ev):
	info = db.data['events'][ev['es_event']]
	for item in db.data['event_messages']:
		if db.data['event_messages'][item]['type'] & info['id']:
			userids = [ev[info['event_var']], ] if info['event_var'] else es.getUseridList()
			delay = info['delay'] + db.data['event_messages'][item]['delay']
			if delay:
				gamethread.delayed(delay,Display,(item, userids, 'event_messages'))
			else:
				Display(item, userids, 'event_messages')

# ######## #
# Commands #
# ######## #

def WebshortcutHandler():
	userid = es.getcmduserid()
	command = es.getargv(0)
	
	url = db.data['webshortcuts'][command]['url']

	if '{NAME}' in url:
		url = url.replace('{NAME}', es.getplayername(userid))
	if '{STEAMID}' in url:
		url = url.replace('{STEAMID}', es.getplayersteamid(userid))
	if '{USERID}' in url:
		url = url.replace('{USERID}', userid)
	if '{IP}' in url:
		url = url.replace('{IP}', es.createplayerlist(userid)[userid]['address'].split(':')[0])
	if '{SERVER_IP}' in url:
		url = url.replace('{SERVER_IP}', db.data['servers']['ip'])
	if '{SERVER_PORT}' in url:
		url = url.replace('{SERVER_PORT}', db.data['servers']['port'])
	if '{LANG}' in url:
		url = url.replace('{LANG}', playerlib.getPlayer(userid).get('lang'))
	
	usermsg.motd(userid, 2, db.data['webshortcuts'][command]['title'], url)

# ####### #
# Classes #
# ####### #
	
class Database(object):
	
	connected = False
	retry = 0
	
	def __init__(self):
		self._connect()
		# Value Sizes
		# TINYINT 255
		# SMALLINT 65535
		# MEDIUMINT 16777215
		# INT 4294967295
		
		# Advertisement Tables
		
		self._execute("""
			CREATE TABLE IF NOT EXISTS `%sadvertisements` (
				`id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL,
				`display` TINYINT UNSIGNED NOT NULL DEFAULT 1,
				`servers` TEXT NOT NULL,
				`order` TINYINT UNSIGNED NOT NULL DEFAULT 128,
				PRIMARY KEY (`id`),
				UNIQUE(`id`)
			) CHARACTER SET = 'utf8';
		""" % (config['MySQL']['prefix']))
		
		self._execute("""
			CREATE TABLE IF NOT EXISTS `%sadvertisement_translations` (
			`id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL,
			`aid` MEDIUMINT UNSIGNED NOT NULL,
			`lang` CHAR(2) NOT NULL DEFAULT 'en',
			`text` TEXT NOT NULL,
			PRIMARY KEY (`id`),
			UNIQUE(`id`)
			) CHARACTER SET = 'utf8';
		""" % (config['MySQL']['prefix']))
		
		# Event Tables & Data
		
		self._execute("""
			CREATE TABLE IF NOT EXISTS `%sevent_messages` (
				`id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL,
				`display` TINYINT UNSIGNED NOT NULL DEFAULT 1,
				`servers` TEXT NOT NULL,
				`order` TINYINT UNSIGNED NOT NULL DEFAULT 128,
				`delay` SMALLINT UNSIGNED NOT NULL DEFAULT 0,
				`type` TINYINT UNSIGNED NOT NULL DEFAULT 0,
				PRIMARY KEY (`id`),
				UNIQUE(`id`)
			) CHARACTER SET = 'utf8';
		""" % (config['MySQL']['prefix']))
		
		self._execute("""
			CREATE TABLE IF NOT EXISTS `%sevent_translations` (
				`id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL,
				`emid` MEDIUMINT UNSIGNED NOT NULL,
				`lang` CHAR(2) NOT NULL DEFAULT 'en',
				`text` TEXT NOT NULL,
				PRIMARY KEY (`id`),
				UNIQUE(`id`)
			) CHARACTER SET = 'utf8';
		""" % (config['MySQL']['prefix']))
		
		# Max Length for events is 32 chars according to modevents.res
		self._execute("""
			CREATE TABLE IF NOT EXISTS `%sevents` (
			`id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL,
			`eid` MEDIUMINT UNSIGNED NOT NULL,
			`event` CHAR(32) NOT NULL,
			`event_var` TEXT NULL,
			`game` TEXT NULL,
			`delay` SMALLINT UNSIGNED NOT NULL DEFAULT 0,
			PRIMARY KEY (`id`),
			UNIQUE (`id`, `event`)
			) CHARACTER SET = 'utf8';
		""" % (config['MySQL']['prefix']))
		
		# Make sure we don't screw up custom settings here
		
		self._execute("SELECT COUNT(*) from %sevents" % config['MySQL']['prefix'])
		if not self.cur.fetchone()[0]:
			self._execute("""
				INSERT INTO %sevents (`eid`, `event`, `event_var`, `game`, `delay`)
				VALUES (1, 'player_connect', 'userid', NULL, 0),
				(2, 'player_activate', 'userid', NULL, 0),
				(3, 'player_death', 'userid', NULL, 0);
			""" % config['MySQL']['prefix'])
		
		# Webshortcuts Table
		
		self._execute("""
			CREATE TABLE IF NOT EXISTS `%swebshortcuts` (
				`id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL,
				`command` TEXT NOT NULL,
				`title` TEXT NOT NULL,
				`url` TEXT NOT NULL,
				`servers` TEXT NOT NULL,
				PRIMARY KEY (`id`),
				UNIQUE(`id`)
			) CHARACTER SET = 'utf8';
		""" % (config['MySQL']['prefix']))
		
		
		# Server Table
		
		self._execute("""
			CREATE TABLE IF NOT EXISTS `%sservers` (
				`id` TINYINT UNSIGNED AUTO_INCREMENT NOT NULL,
				`ip` CHAR(15) NOT NULL,
				`port` SMALLINT UNSIGNED NOT NULL,
				`interval` SMALLINT UNSIGNED NOT NULL DEFAULT 30,
				`dateformat` CHAR(10) NOT NULL DEFAULT '%%d.%%m.%%Y',
				`timeformat` CHAR(10) NOT NULL DEFAULT '%%H:%%M:%%S',
				`random` BOOLEAN NOT NULL DEFAULT FALSE,
				PRIMARY KEY (`id`),
				UNIQUE(`id`)
			) CHARACTER SET = 'utf8';
		""" % (config['MySQL']['prefix']))
		
		#
		self.data = {}
		self.data['servers'] = {}
		self.data['advertisements'] = {}
		self.data['advertisements']['list'] = []
		self.data['events'] = {}
		self.data['event_messages'] = {}
		self.data['webshortcuts'] = {}
		
		# Syncronize DB
		self.SyncDB()
	def _connect(self):
		try:
			self.con = mysql.connect(host=config['MySQL']['server'], user=config['MySQL']['user'], passwd=config['MySQL']['password'], db=config['MySQL']['database'], connect_timeout=int(config['MySQL']['timeout']))
		#PY 2.6: except OperationalError as error
		except OperationalError:
			# Handling for failure to connect to MySQL server
			error = sys.exc_info()[1]
			if error[0] == 2003:
				Debug(error[1])
				self.connected = False
				if self.retry >= config['ErrorHandling']['Retry']:
					if config['ErrorHandling']['RetryBehaviour'] == '0':
						es.unload('mysqlads')
				else:
					self.retry += 1
					self._connect()
			else:
				raise OperationalError(error[0],error[1])
		else:
			self.retry = 0
			self.connected = True
			self.cur = self.con.cursor()
			self._execute("SET CHARACTER SET 'utf8'")
	def _execute(self, command):
		try:
			self.cur.execute(command)
		except OperationalError:
			# Fix for "MySQL Server has gone away"
			error = sys.exc_info()[1]
			if error[0] == 2006:
				Debug(error[1])
				self._connect()
				self._execute(command)
			else:
				raise OperationalError(error[0],error[1])
	def SyncDB(self):
		if not self.connected:
			self._connect()
		Debug('Starting database syncronisation!')
		
		
		
		# Get ServerID and configuration vars
		if config['MySQL']['sid'] == '-1':
			global ip, port
			self._execute('SELECT `id`, `interval`, `dateformat`, `timeformat`, `random`, `ip`, `port` FROM %sservers WHERE `ip` = "%s" AND `port` = "%s" LIMIT 1' % (config['MySQL']['prefix'], ip, port))
		else:
			self._execute('SELECT `id`, `interval`, `dateformat`, `timeformat`, `random`, `ip`, `port` FROM %sservers WHERE `id` = "%s" LIMIT 1' % (config['MySQL']['prefix'], config['MySQL']['sid']))
		data = self.cur.fetchone()
		if data:
			self.data['servers'] = {}
			self.data['servers']['sid'] = data[0]
			self.data['servers']['interval'] = data[1]
			self.data['servers']['dateformat'] = data[2]
			self.data['servers']['timeformat'] = data[3]
			self.data['servers']['random'] = data[4]
			self.data['servers']['ip'] = data[5]
			self.data['servers']['port'] = data[6]
			
			# Get all valid advertisements
			self._execute("""
				SELECT `id`, `display`
				FROM `%sadvertisements`
				WHERE LOCATE('!', servers) !=0 AND LOCATE('%s,', servers ) = 0
					OR LOCATE('!', servers) = 0 AND (LOCATE('-1,', servers) != 0 OR LOCATE('%s,',servers) != 0)
				ORDER BY `order`
				""" % (config['MySQL']['prefix'],self.data['servers']['sid'],self.data['servers']['sid']))
				
			data = self.cur.fetchall()
			if data:
				self.data['advertisements'] = {}
				self.data['advertisements']['list'] = []
				for item in data:
					self.data['advertisements']['list'].append(item[0])
					self.data['advertisements'][item[0]] = {}
					self.data['advertisements'][item[0]]['display'] = item[1]
					self.data['advertisements'][item[0]]['translations'] = {}
					# Get all corresponding translations
					self._execute("SELECT lang, text FROM %sadvertisement_translations WHERE aid = '%s'" % (config['MySQL']['prefix'], item[0]))
					tdata = self.cur.fetchall()
					for translation in tdata:
						self.data['advertisements'][item[0]]['translations'][translation[0]] = translation[1]	
			else:
				Debug('No advertisements for this server found!')
			
			
			# Get all events and validate them
			oldevents = self.data['events']
			self.data['events'] = {}
			
			self._execute("""
				SELECT `eid`, `event`, `event_var`, `delay`
				FROM `%sevents`
				WHERE `game` IS NULL OR `game` = '%s'
				""" % (config['MySQL']['prefix'], es.getString('eventscripts_gamedir').split('/')[-1]))
				
			eventdata = self.cur.fetchall()
			if eventdata:
				for event in eventdata:
					# TODO: Check for event existance
					self.data['events'][event[1]] = {'id': event[0], 'event_var': event[2], 'delay': event[3]}
					#else:
					#	Debug('Event "%s" does not exist!' % event[1])
				
				self._execute("""
				SELECT `id`, `display`, `delay`, `type`
				FROM `%sevent_messages`
				WHERE LOCATE('!', servers) !=0 AND LOCATE('%s,', servers ) = 0
					OR LOCATE('!', servers) = 0 AND (LOCATE('-1,', servers) != 0 OR LOCATE('%s,',servers) != 0)
				ORDER BY `order`
				""" % (config['MySQL']['prefix'],self.data['servers']['sid'],self.data['servers']['sid']))
					
				data = self.cur.fetchall()
				if data:
					self.data['event_messages'] = {}
					for item in data:
						self.data['event_messages'][item[0]] = {}
						self.data['event_messages'][item[0]]['display'] = item[1]
						self.data['event_messages'][item[0]]['delay'] = item[2]
						self.data['event_messages'][item[0]]['type'] = item[3]
						self.data['event_messages'][item[0]]['translations'] = {}
						# Get all corresponding translations
						self._execute("SELECT lang, text FROM %sevent_translations WHERE emid = '%s'" % (config['MySQL']['prefix'], item[0]))
						tdata = self.cur.fetchall()
						for translation in tdata:
							self.data['event_messages'][item[0]]['translations'][translation[0]] = translation[1]
				else:
					Debug('No event messages for this server found!')
			else:
				Debug('No events found. Check your database!')
			
			# Make sure we only listen to the events we need to listen to
			if not oldevents == self.data['events']:
				for event in self.data['events']:
					if not oldevents.has_key(event):
						es.addons.registerForEvent(selfmodule, event, CustomEvent)
				for event in oldevents:
					if not self.data['events'].has_key(event):
						es.addons.unregisterForEvent(selfmodule, event)
			
			
			# Webshortcuts
			oldwebshortcuts = self.data['webshortcuts']
			self.data['webshortcuts'] = {}
			
			self._execute("""
				SELECT `command`, `title` , `url`
				FROM %swebshortcuts
				WHERE LOCATE('!', servers) !=0 AND LOCATE('%s,', servers ) = 0
					OR LOCATE('!', servers) = 0 AND (LOCATE('-1,', servers) != 0 OR LOCATE('%s,',servers) != 0)
				""" % (config['MySQL']['prefix'],self.data['servers']['sid'],self.data['servers']['sid']))
			
			data = self.cur.fetchall()
			if data:
				for item in data:
					for command in item[0].strip(',').split(','):
						self.data['webshortcuts'][command] = {'title': item[1], 'url': item[2]}
			else:
				Debug('No webshortcuts for this server found!')
			
			# Make sure we have the commands registered we need to have registered ;)
			if not oldevents == self.data['webshortcuts']:
				for command in self.data['webshortcuts']:
					if not oldwebshortcuts.has_key(command):
						es.regsaycmd(command, 'mysqlads/WebshortcutHandler', 'Webshortcut')
				for command in oldevents:
					if not self.data['webshortcuts'].has_key(command):
						es.unregsaycmd(command)

			Debug('Database syncronisation completed!')
		else:
			if config['MySQL']['sid'] == '-1':
				Debug('Server not found in database (IP: %s Port %s). Unloading.' % (ip,port))
			else:
				Debug('Server not found in database (SID: %s). Unloading.' % sid)
			es.unload('mysqlads')
	def Disconnect(self):
		for event in self.data['events']:
			es.addons.unregisterForEvent(selfmodule, event)
		if self.connected:
			self.cur.close()
			self.con.close()
		
# ######### #
# Functions #
# ######### #

def AdLoop(aid=-1):
	if db.data['advertisements']['list']:
		if db.data['servers']['random']:
			aid = db.data['advertisements']['list'][random.randint(0,len(db.data['advertisements']['list'])) - 1]
		else:
			if aid == -1:
				index = 0
			else:
				index = db.data['advertisements']['list'].index(aid) + 1
				if index >= len(db.data['advertisements']['list']):
					index = 0
			aid = db.data['advertisements']['list'][index]
		Display(aid, es.getUseridList())
	gamethread.delayedname(db.data['servers']['interval'], 'mysqlads_adloop', AdLoop, aid) 
	
def Display(id, userids, type='advertisements'):
	"""
	Modes
	1 = Chat 
	2 = Menu
	4 = Hint
	8 = Center
	"""
	mode = db.data[type][id]['display']
	if mode:
		for userid in userids:
			lng = playerlib.getPlayer(userid).get('lang')
			
			# Any valid message for this achievement?
			if db.data[type][id]['translations'].has_key(lng):
				text = db.data[type][id]['translations'][lng]
			elif db.data[type][id]['translations'].has_key('en'):
				text = db.data[type][id]['translations']['en']
			else:
				text = lang('missing translation',{'aid': id},lng)
			
			# Parse variables
			if '{CURRENTMAP}' in text:
				text = text.replace('{CURRENTMAP}',es.getString('eventscripts_currentmap'))
			if '{DATE}' in text:
				text = text.replace('{DATE}',time.strftime(db.data['servers']['dateformat']))
			if '{TIME}' in text:
				text = text.replace('{TIME}',time.strftime(db.data['servers']['timeformat']))
			cvars = re.findall(r'\{CVAR:[^\}]*\}',text)
			if cvars:
				for cvar in cvars:
					text = text.replace(cvar,es.getString(cvars[cvars.index(cvar)][6:-1]))
			
			if mode & 1:
				es.tell(userid,'#multi',text)
			# Following modes don't use the chat colours
			for color in ['#green','#lightgreen','#default']:
				matches = re.findall(color, text, re.IGNORECASE)
				for match in matches:
					text = text.replace(match,'')
			if mode & 2:
				es.menu(10,userid,text)
			if mode & 4:
				usermsg.hudhint(userid,text)
			if mode & 8:
				usermsg.centermsg(userid,text)

def Debug(msg):
	es.dbgmsg(0,'mysqlads.py: ' + msg)