"""
addons/eventscripts/update/update.py
For mysqlads 1.02.000
(C) [#OMEGA] - K2

IMPORTANT

This plugin is designed to be used with
MySQLdb or pymysql

Agreement:
 
By using this file you agree to the following terms:

This file is delivered 'as-is'. 
I will not take resposibilty for any damage resulting of the use or not-use of this file. 

As for sharing permissions:
http://creativecommons.org/licenses/by-nc-nd/3.0/

If wish to alter the file/contents or reuse some parts please contact me.
"""

# ####### #
# Imports #
# ####### #

# ES Libs
import es

# Python Libs
import re
from configobj import ConfigObj

# Custom Python Libs
try:
	import MySQLdb as mysql
except:
	try:
		import pymysql as mysql
	except:
		raise ImportError, "Failed to import MySQLdb or pymysql."

# ########## #
# Addon Info #
# ########## #

info = es.AddonInfo()
info.name        = "MySQL Advertisements - Updater"
info.version     = "1.02.000"
info.author      = "[#OMEGA] - K2"
info.url         = "http://www.german-slaughterhouse.de"
info.description = "Updater for MySQL Advertisements"

# ############# #
# Load & Unload #
# ############# #

def load():
	es.regcmd('mysqlads_update','mysqlads/update/Update','mysqlads_update [version] - Updates Config and Datebase to the latest verisons (assuming the last released version is used). If version is specified, it will be updated from coresponding version')

# ######## #
# Commands #
# ######## #

def Update():
	# Verify Arguments First
	argc = es.getargc()
	if argc == 1:
		version = '1.01.002'
	elif argc == 2:
		version = es.getargs()[0]
		if not re.findall('^[1-9]\.[0-9]{2}\.[0-9]{3}$', version):
			raise UpdateError('Invalid version number specified!')
	else:
		raise UpdateError('Invalid amount of arguments specified!')
	
	newver = ConvertVersionNumber(info.version)
	
	# Load & Update Config
	config = ConfigObj(es.getAddonPath("mysqlads") + '/config.ini')
	
	ver = ConvertVersionNumber(version)
	while (ver != newver):
		if ver[0] == 1:
			# Versions 1.00.000, 1.01.000, 1.01.001 and 1.01.002
			if ver[1] <= 1:
				config['MySQL']['timeout'] = 10
				config['ErrorHandling']['Retry'] = 5
				config['ErrorHandling']['RetryBehaviour'] = 1
				config.write()
				ver[1] = 2
				ver[2] = 0
	
		es.dbgmsg(0, 'mysqlads/update.py: Config updated to version %i.%02i.%03i' % tuple(ver))
	
	# Database Updates
	# New tables won't be added during the progress, as they are automatically added by the main script
	con = mysql.connect(host=config['MySQL']['server'], user=config['MySQL']['user'], passwd=config['MySQL']['password'], db=config['MySQL']['database'])
	cur = con.cursor()
	
	ver = ConvertVersionNumber(version)
	while (ver != newver):
		if ver[0] == 1:
			# Version 1.00.000
			if ver[1] == 0:
				cur.execute("""
					ALTER TABLE `%sadvertisements` 
						CONVERT TO CHARACTER SET 'utf8',
						DEFAULT CHARACTER SET 'utf8';
				""" % (config['MySQL']['prefix']))
				cur.execute("""
					ALTER TABLE `%stranslations`
						CONVERT TO CHARACTER SET 'utf8',
						DEFAULT CHARACTER SET 'utf8';
				""" % (config['MySQL']['prefix']))
				cur.execute("""
					ALTER TABLE `%sservers`
						CONVERT TO CHARACTER SET 'utf8',
						DEFAULT CHARACTER SET 'utf8';
				""" % (config['MySQL']['prefix']))
				ver[1] = 1
			# Versions 1.01.000, 1.01.001 and 1.01.002
			elif ver[1] == 1:
				cur.execute("""
					ALTER TABLE `%sadvertisements`
						CHANGE COLUMN `aid` `id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL,
						DROP PRIMARY KEY,
						ADD PRIMARY KEY (`id`),
						ADD UNIQUE (`id`);
				""" % (config['MySQL']['prefix']))

				cur.execute("""
					ALTER TABLE `%stranslations`
						RENAME TO `%sadvertisement_translations`,
						ADD COLUMN `id` MEDIUMINT UNSIGNED AUTO_INCREMENT NOT NULL FIRST,
						ADD PRIMARY KEY (`id`),
						ADD UNIQUE (`id`);
				""" % (config['MySQL']['prefix'],config['MySQL']['prefix']))
				
				cur.execute("""
					ALTER TABLE `%sservers`
						CHANGE COLUMN `sid` `id` TINYINT UNSIGNED NOT NULL AUTO_INCREMENT,
						DROP PRIMARY KEY,
						ADD PRIMARY KEY (`id`),
						ADD UNIQUE (`id`);
				""" % (config['MySQL']['prefix']))
				ver[1] = 2
				ver[2] = 0
		es.dbgmsg(0, 'mysqlads/update.py: Database updated to version %i.%02i.%03i' % tuple(ver))
	cur.close()
	con.close()

#
# Functions
#

def ConvertVersionNumber(version):
	version = version.split('.')
	for i in range(0,3):
		version[i] = int(version[i])
	return version

# ####### #
# Classes #
# ####### #

class UpdateError(Exception):
	pass